import os
import subprocess
import customtkinter as ctk
from tkinter import messagebox, Menu, Tk
from yt_dlp import YoutubeDL
from threading import Thread
import re

# הגדרות ראשוניות
ctk.set_appearance_mode("Light")  # אפשרויות: "Light", "Dark", "System"
ctk.set_default_color_theme("blue")  # צבע ברירת מחדל

# תיקיית יעד
download_folder = os.getcwd()

# יצירת חלון ראשי
root = ctk.CTk()
root.title("מוריד סרטונים מיוטיוב")
root.geometry("550x550")  # אורך חלון מותאם כך שיתאימו כל האלמנטים
root.resizable(False, False)

# פונקציה לעדכון yt-dlp
def update_yt_dlp():
    try:
        subprocess.run(["python", "-m", "pip", "install", "--upgrade", "yt-dlp"], check=True)
        messagebox.showinfo("העדכון הושלם", "הגרסה של yt-dlp מעודכנת להכי חדשה!")
    except subprocess.CalledProcessError:
        messagebox.showerror("שגיאה", "ההתקנה או העדכון של yt-dlp נכשל.")

# פונקציה לפתיחת תיקיית היעד
def open_download_folder():
    try:
        Thread(target=lambda: subprocess.Popen(f'explorer "{download_folder}"')).start()
    except Exception as e:
        messagebox.showerror("שגיאה", f"לא ניתן לפתוח את התיקיה: {str(e)}")

# יצירת תפריט "אפשרויות"
menu_bar = Menu(root)
options_menu = Menu(menu_bar, tearoff=0)
options_menu.add_command(label="פתח תיקיית יעד", command=open_download_folder)
options_menu.add_command(label="עדכן yt-dlp", command=update_yt_dlp)

# פונקציה לפתיחת חלון אודות
def open_help_window():
    help_window = ctk.CTkToplevel(root)  # יצירת חלון חדש
    help_window.title("אודות")
    help_window.geometry("500x300")

    help_text = """ הממשק פותח על ידי אלחנן
הזכויות אינן שמורות, העתק והעבר כרצונך
אתה מוזמן גם לשנות את הקוד כאוות נפשך
בהצלחה
    """
    
    help_label = ctk.CTkLabel(help_window, text=help_text, font=("Arial", 12))
    help_label.pack(padx=20, pady=20)

    # כפתור אישור
    ok_button = ctk.CTkButton(help_window, text="אישור", command=help_window.destroy, fg_color="lightgreen", corner_radius=8)
    ok_button.pack(pady=10)

options_menu.add_command(label="אודות", command=open_help_window)
menu_bar.add_cascade(label="אפשרויות", menu=options_menu)

# הגדרת תפריט החלון
root.config(menu=menu_bar)

# פונקציה להורדת סרטון
def download_video():
    url = url_entry.get()
    if not url:
        status_label.configure(text="שגיאה: אנא הזן כתובת URL", text_color="red")
        return

    quality = quality_combobox.get()
    if not quality:
        status_label.configure(text="שגיאה: אנא בחר איכות וידיאו או פסקול", text_color="red")
        return

    progress_text.configure(state="normal")
    progress_text.delete("1.0", "end")
    status_label.configure(text="מכין להורדה...", text_color="blue")
    progress_bar.set(0)

    def clean_ansi_codes(text):
        return re.sub(r'\x1b\[[0-9;]*m', '', text)

    def hook(d):
        if d['status'] == 'downloading':
            percent_str = clean_ansi_codes(d.get('_percent_str', '0').strip('%'))
            try:
                percent = float(percent_str)
                progress_bar.set(percent / 100)
                progress_label.configure(text=f"התקדמות: {percent}%")
            except ValueError:
                progress_bar.set(0)

            title = clean_ansi_codes(d['info_dict'].get('title', 'לא ידוע'))
            speed = clean_ansi_codes(d.get('_speed_str', 'לא זמין'))
            eta = clean_ansi_codes(d.get('_eta_str', 'לא זמין'))
            progress_percent = clean_ansi_codes(d.get('_percent_str', '0%'))

            progress_text.delete("1.0", "end")
            progress_text.insert("end", f"קובץ: {title}\nמהירות: {speed}\nזמן שנותר: {eta}\nהתקדמות: {progress_percent}")

    quality_dict = {
        '144': 'best[height<=144]',
        '360': 'best[height<=360]',
        '480': 'best[height<=480]',
        '720': 'best[height<=720]',
        '1080': 'best[height<=1080]',
        'Audio Only': 'bestaudio[ext=m4a]/bestaudio'
    }

    selected_format = quality_dict.get(quality, 'best')

    ydl_opts = {
        'format': selected_format,
        'progress_hooks': [hook],
        'outtmpl': os.path.join(download_folder, '%(title)s.%(ext)s'),
        'postprocessors': [{
            'key': 'FFmpegVideoConvertor',
            'preferedformat': 'mp4',  # אפשר לשנות לפורמט אחר אם צריך
        }],
        'nocheckcertificate': True  # התעלמות מבעיות אישור SSL
    }

    def run_download():
        try:
            with YoutubeDL(ydl_opts) as ydl:
                ydl.download([url])
            status_label.configure(text="ההורדה הושלמה!", text_color="green")
            messagebox.showinfo("ההורדה הושלמה", "ההורדה הושלמה בהצלחה!")
        except Exception as e:
            status_label.configure(text=f"שגיאה: {str(e)}", text_color="red")
            messagebox.showerror("שגיאה בהורדה", f"שגיאה: {str(e)}")

    Thread(target=run_download).start()

# פונקציה להדבקת URL ובדיקתו
def paste_and_validate_url():
    try:
        url = root.clipboard_get()
        url_entry.delete(0, "end")
        url_entry.insert(0, url)
        if not re.match(r'https?://(?:www\.)?youtube\.com/watch\?v=', url):
            messagebox.showwarning("אזהרה", "ייתכן שהכתובת שהודבקה אינה כתובת YouTube חוקית. אנא בדוק שוב.")
    except:
        messagebox.showerror("שגיאה", "לוח ההדבקה ריק או אינו מכיל טקסט.")

# כותרת
title_label = ctk.CTkLabel(root, text="המוריד", font=("Arial", 24, "bold"), text_color="darkblue")
title_label.pack(pady=20)

# שדה URL
url_frame = ctk.CTkFrame(root)
url_frame.pack(pady=10)

url_entry = ctk.CTkEntry(url_frame, width=350, placeholder_text="יוטיוב סרטון כתובת הכנס", corner_radius=10)
url_entry.pack(side="left", padx=10)

# כפתור הדבקה
paste_button = ctk.CTkButton(url_frame, text="📋", command=paste_and_validate_url, fg_color="lightgray", width=40, height=40, corner_radius=8, text_color="darkorange", font=("Arial", 20, "bold"))
paste_button.pack(side="left", padx=5)

# בחירת איכות וידיאו
quality_combobox = ctk.CTkComboBox(root, values=['144', '360', '480', '720', '1080', 'Audio Only'], width=200, corner_radius=8)
quality_combobox.set('720')
quality_combobox.pack(pady=10)

# כפתור הורדה
download_button = ctk.CTkButton(root, text="הורד", command=download_video, fg_color="green", width=150, corner_radius=8)
download_button.pack(pady=20)

# סרגל התקדמות
progress_label = ctk.CTkLabel(root, text="התקדמות:", font=("Arial", 14))
progress_label.pack(pady=5)

progress_bar = ctk.CTkProgressBar(root, width=400, height=20)
progress_bar.pack(pady=10)

# תיבת טקסט להודעות
progress_text = ctk.CTkTextbox(root, width=400, height=150, wrap="word", state="normal")
progress_text.pack(pady=20)

# כותרת למצב
status_label = ctk.CTkLabel(root, text="", font=("Arial", 16))
status_label.pack(pady=10)

# הפעלת התוכנית
root.mainloop()
